<?php
/*
Plugin Name: CSS And Script File Aggregation
Plugin URI: http://yacobi.info/
Description: Aggregates css and js files to minimize HTTP requests sent from browsers and speed up page load.
Version: 1.0
Author: Ronen Yacobi
Author URI: http://yacobi.info/
License:  GPL2
*/

if (!is_admin()) {
	add_action('wp_print_styles', 'combine_css_files',99999);
	add_action('wp_print_scripts', 'combine_js_files',99999);
}
else {
	add_filter('plugin_row_meta', 'RegisterPluginLinks',10,2);
}

function RegisterPluginLinks($links,$file) {
	 if ( $file == plugin_basename(__FILE__) ) {
		 $links[] = '<a href="https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=7KGK3D9TRDF6E">Like it? Buy me a beer</a>';
	 }
	 return $links;
}
	
function combine_css_files() {
	global $wp_styles;
	define('CSS_BASE_NAME','combined-style-');
	$faoptions = get_option('faoptions');
	if (!is_array($faoptions)) $faoptions=array();
	$css_files_modify=array();
	$css_files=array();
	if (is_array($wp_styles->queue)) {  //is there anything in the queue?
		$wp_styles->all_deps($wp_styles->queue); //preparing the queue taking dependencies into account
		foreach ($faoptions As $css => $value) { //going through all the already found css files, checking that they are still required
			if ((!in_array(substr($css,4),$wp_styles->to_do)) && substr($css,0,4)=='css-') {  //if the css is not queued, rewrite the file
				$css_media=$value['type'];
				$css_files_modify[$css_media] = true;
				unset($faoptions[$css]);
			}
		}
		foreach ($wp_styles->to_do As $css) {
			$css_src=$wp_styles->registered[$css]->src;
			$css_media=$wp_styles->registered[$css]->args;
			if (!(strpos($css_src,get_bloginfo('url'))===false)  || substr($css_src,0,1)==="/" || substr($css_src,0,1)===".") { //the css is hosted localy
				if (!is_array($css_files[$css_media])) {
					$css_files[$css_media]=array();
				}
				if (strpos($css_src,get_bloginfo('url'))===false) {
					$css_relative_url=substr($css_src,1);
				}
				else {
					$css_relative_url=substr($css_src,strlen(get_bloginfo('url'))+1);
				}	
				array_push($css_files[$css_media],$css_relative_url);
				$css_m_time=filemtime($css_relative_url); //getting the mofified time of the css file. extracting the file's dir
				if ( (!file_exists(dirname(__FILE__)."/". CSS_BASE_NAME .$css_media.".css")) //combined css doesn't exist
					|| (isset($faoptions['css-'.$css]) && (($css_m_time <> $faoptions['css-'.$css]['modified']) || $faoptions['css-'.$css]['type']<>$css_media )) //css file has changed or the media type changed
					|| (!isset($faoptions['css-'.$css]))  ) { //css file is first identified
						$css_files_modify[$css_media] = true;  //the combined file containing this media type css should be changed
						if (isset($faoptions['css-'.$css]) && $faoptions['css-'.$css]['type']<>$css_media) { //if the media type changed - rewrite both css files
							$tmp=$faoptions['css-'.$css]['type'];
							$css_files_modify[$tmp] = true;
						}
						if (!is_array($faoptions['css-'.$css])) $faoptions['css-'.$css]=array();
						$faoptions['css-'.$css]['modified'] = $css_m_time; //write the new modified date
						$faoptions['css-'.$css]['type'] = $css_media;
				}
				$wp_styles->remove($css);  //removes the css file from the queue
			}
		}
	}
	foreach ($css_files_modify As $key => $value) {
		$combined_file=fopen(dirname(__FILE__)."/". CSS_BASE_NAME .$key.".css", 'w') or die("ERROR OPENING CSS FILE");
		$css_content='';
		if (is_array($css_files[$key])) {
			foreach ($css_files[$key] As $css_src) {
				$source_file=fopen($css_src,'r') or die("ERROR OPENING CSS FILE");
				$css_content .= "/* ". $css_src." content: */\n" . fread($source_file,filesize($css_src))."\n";
				fclose($source_file);
			}
		}
		if (!isset($faoptions['ver'][$key])) $faoptions['ver'][$key]=0;
		$faoptions['ver'][$key]++;
		fwrite($combined_file,$css_content);
		fclose($combined_file);
	}
	foreach ($css_files As $key => $value) { //enqueue the combined css files
		wp_enqueue_style(CSS_BASE_NAME.$key,WP_CONTENT_URL.'/plugins/'.plugin_basename(dirname(__FILE__))."/". CSS_BASE_NAME.$key.".css", array(), $faoptions['ver'][$key], $key );
	}
	update_option('faoptions',$faoptions);
}

function combine_js_files() {
	global $wp_scripts;
	global $skip_combine_js;
	if (isset($skip_combine_js)) return null; //Don't run twice
	$skip_combine_js=true;
	$faoptions = get_option('faoptions');
	if (!is_array($faoptions)) $faoptions=array();
	$js_files_modify=array();
	$js_files=array();
	if (is_array($wp_scripts->queue)) {  //is there anything in the queue?
		$wp_scripts->all_deps($wp_scripts->queue); //preparing the queue taking dependencies into account
		foreach ($faoptions As $js => $value) { //going through all the already found js files, checking that they are still required
			if ((!in_array(substr($js,3),$wp_scripts->to_do)) && substr($js,0,3)=='js-') {  //if the js is not queued, rewrite the file
				$js_place=$value['type'];
				$js_files_modify[$js_place] = true;
				unset($faoptions[$js]);
			}
		}
		foreach ($wp_scripts->to_do As $js) {
			$js_src=$wp_scripts->registered[$js]->src;
			$js_place=$wp_scripts->registered[$js]->extra;
			if (is_array($js_place) && $js_place['group']==1) {
				$js_place='footer';
			}
			else {
				$js_place='header';
			}
			if (!(strpos($js_src,get_bloginfo('url'))===false) || substr($js_src,0,1)==="/" || substr($js_src,0,1)===".") { //the js is hosted localy
				if (!is_array($js_files[$js_place])) {
					$js_files[$js_place]=array();
				}
				if (strpos($js_src,get_bloginfo('url'))===false) {
					$js_relative_url=substr($js_src,1);
				}
				else {
					$js_relative_url=substr($js_src,strlen(get_bloginfo('url'))+1);
				}	
				array_push($js_files[$js_place],$js_relative_url);
				$js_m_time=filemtime($js_relative_url); //getting the mofified time of the js file. extracting the file's dir
				if ( (!file_exists(dirname(__FILE__)."/combined-scripts-".$js_place.".js")) //combined js doesn't exist
					|| (isset($faoptions['js-'.$js]) && (($js_m_time <> $faoptions['js-'.$js]['modified']) || $faoptions['js-'.$js]['type']<>$js_place )) //js file has changed or the target place changed
					|| (!isset($faoptions['js-'.$js]))  ) { //js file is first identified
						$js_files_modify[$js_place] = true;  //the combined file containing this place js should be changed
						if (isset($faoptions['js-'.$js]) && $faoptions['js-'.$js]['type']<>$js_place) { //if the place type changed - rewrite both js files
							$tmp=$faoptions['js-'.$js]['type'];
							$js_files_modify[$tmp] = true;
						}
						if (!is_array($faoptions['js-'.$js])) $faoptions['js-'.$js]=array();
						$faoptions['js-'.$js]['modified'] = $js_m_time; //write the new modified date
						$faoptions['js-'.$js]['type'] = $js_place;
				}
				$wp_scripts->remove($js);  //removes the js file from the queue
				array_shift($wp_scripts->to_do);
			}
		}
	}
	foreach ($js_files_modify As $key => $value) {
		$combined_file=fopen(dirname(__FILE__)."/combined-scripts-" .$key.".js", 'w') or die("ERROR OPENING CSS FILE");
		$js_content='';
		if (is_array($js_files[$key])) {
			foreach ($js_files[$key] As $js_src) {
					$source_file=fopen($js_src,'r') or die("ERROR OPENING JS FILE");
					$js_content .= "/* ". $js_src." content: */\n" . fread($source_file,filesize($js_src))."\n";
					fclose($source_file);
			}
		}
		if (!isset($faoptions['ver'][$key."-js"])) $faoptions['ver'][$key."-js"]=0;
		$faoptions['ver'][$key."-js"]++;
		fwrite($combined_file,$js_content);
		fclose($combined_file);
	}
	//enqueue the combined js files
	if (isset($js_files["header"])) {
		wp_enqueue_script('fa-combined-header',WP_CONTENT_URL.'/plugins/'.plugin_basename(dirname(__FILE__))."/combined-scripts-header.js", array(), $faoptions['ver']["header-js"], false );
	}
	if (isset($js_files["footer"])) {
		wp_enqueue_script('fa-combined-footer',WP_CONTENT_URL.'/plugins/'.plugin_basename(dirname(__FILE__))."/combined-scripts-footer.js", array(), $faoptions['ver']["footer-js"], true );
	}
	update_option('faoptions',$faoptions); 
}
?>